package com.mcwl.communityCenter.webSocket;

import com.mcwl.communityCenter.service.AIService;
import com.mcwl.communityCenter.service.CustomerService;
import lombok.NoArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.socket.CloseStatus;
import org.springframework.web.socket.TextMessage;
import org.springframework.web.socket.WebSocketSession;
import org.springframework.web.socket.handler.AbstractWebSocketHandler;
import reactor.core.Disposable;
import reactor.core.publisher.Flux;

import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@ServerEndpoint("/chat")
@NoArgsConstructor
public class ChatWebSocketPoint extends AbstractWebSocketHandler {
    private final Map<String, Boolean> userModes = new ConcurrentHashMap<>();

    // 存储会话与订阅的映射关系
    private final Map<String, Disposable> sessionSubscriptions = new ConcurrentHashMap<>();

    @Autowired
    private AIService aiService;

    @Autowired
    private CustomerService customerService;

    // 构造函数注入服务...

    @Override
    public void handleTextMessage(WebSocketSession session, TextMessage message) throws Exception {
        String userId = session.getId();
        String userMessage = message.getPayload();
        // 判断当前模式
        if (userModes.getOrDefault(userId, false)) {
            // 人工模式
            customerService.handleCustomerMessage(userId, userMessage);
        } else {
            // AI 流式响应模式
            Flux<String> responseStream = aiService.getDeepSeekResponseStream(userMessage);

            // 订阅响应流并存储 Disposable
            Disposable disposable = responseStream
                    .doOnNext(chunk -> sendText(session, chunk))
                    .doOnComplete(() -> sendText(session, "[END]"))
                    .doOnError(e -> sendText(session, "[ERROR] " + e.getMessage()))
                    .subscribe();

            sessionSubscriptions.put(userId, disposable);
        }
    }

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        super.afterConnectionEstablished(session);
        userModes.put(session.getId(), false);
    }

    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus status) {
        // 清理订阅资源
        String sessionId = session.getId();
        Disposable disposable = sessionSubscriptions.remove(sessionId);
        if (disposable != null && !disposable.isDisposed()) {
            disposable.dispose();
            System.out.println("已清理会话资源: " + sessionId);
        }
    }

    // 添加模式切换方法（根据业务需求）
    public void switchUserMode(String sessionId, boolean isHumanMode) {
        userModes.put(sessionId, isHumanMode);
    }

    // 线程安全的发送方法
    private void sendText(WebSocketSession session, String text) {
        try {
            if (session.isOpen()) {
                synchronized (session) { // WebSocketSession 非线程安全
                    session.sendMessage(new TextMessage(text));
                }
            }
        } catch (IOException e) {
            System.out.println("WebSocket 发送失败: " + e.getMessage());
        }
    }
}